import { MultiGraph } from 'graphology';

export function DragAttributePillStarted(id: string, nodes: MultiGraph) {
  // if the attribute is still connected to an entity or relation pill, disconnect
  const es = nodes.outEdges(id);
  es.forEach((e) => nodes.dropEdge(e));
}

export function DragAttributePill(
  id: string,
  nodes: MultiGraph,
  dx: number,
  dy: number
) {
  // Get the closes entity or relation node
  const closestNode = GetClosestPill(id, nodes, ['entity', 'relation']);
  // If we found one, highlight it by adding an attribute
  if (closestNode)
    nodes.setNodeAttribute(closestNode, 'suggestedForConnection', true);
}

export function DragAttibutePillStopped(id: string, nodes: MultiGraph) {
  // If there is currently a node with the suggestedForConnection attribute
  // connect this attribute to it
  nodes.forEachNode((node, { suggestedForConnection }) => {
    if (suggestedForConnection)
      nodes.addEdge(id, node, { type: 'attribute_connection' });
  });
}

/**
 * Gets the closest node to id
 * @param id
 * @param nodes Graphology querybuilder MultiGraph object
 * @param allowedNodeTypes An array of the node types which are included in the search
 * @param maxDistance The maximum distance
 * @returns the closest node if within range
 */
function GetClosestPill(
  id: string,
  nodes: MultiGraph,
  allowedNodeTypes: string[],
  maxDistance = 150
): string | undefined {
  const { x, y, w, h } = nodes.getNodeAttributes(id);
  const center: { x: number; y: number } = { x: x + w / 2, y: y + h / 2 };

  let minDist = maxDistance * maxDistance;
  let closestNode: string | undefined = undefined;
  nodes.forEachNode((node, { x, y, w, h, type }) => {
    if (allowedNodeTypes.includes(type)) {
      const nodeCenter: { x: number; y: number } = {
        x: x + w / 2,
        y: y + h / 2,
      };

      const dx = center.x - nodeCenter.x;
      const dy = center.y - nodeCenter.y;
      const dist = dx * dx + dy * dy;
      if (dist < minDist) {
        minDist = dist;
        closestNode = node;
      }
    }
  });

  return closestNode;
}
